@@ -3,21 +3,11 @@ package aghtest
 
 import (
 	"io"
-	"os"
 	"testing"
 
 	"github.com/AdguardTeam/golibs/log"
 )
 
-// DiscardLogOutput runs tests with discarded logger output.
-func DiscardLogOutput(m *testing.M) {
-	// TODO(e.burkov): Refactor code and tests to not use the global mutable
-	// logger.
-	log.SetOutput(io.Discard)
-
-	os.Exit(m.Run())
-}
-
 // ReplaceLogWriter moves logger output to w and uses Cleanup method of t to
 // revert changes.
 func ReplaceLogWriter(t testing.TB, w io.Writer) {
