@@ -4,7 +4,6 @@
 package tools
 
 import (
-	// Tools.
 	_ "github.com/fzipp/gocyclo/cmd/gocyclo"
 	_ "github.com/golangci/misspell/cmd/misspell"
 	_ "github.com/gordonklaus/ineffassign"
@@ -13,6 +12,7 @@ import (
 	_ "github.com/securego/gosec/v2/cmd/gosec"
 	_ "golang.org/x/tools/go/analysis/passes/nilness/cmd/nilness"
 	_ "golang.org/x/tools/go/analysis/passes/shadow/cmd/shadow"
+	_ "golang.org/x/vuln/cmd/govulncheck"
 	_ "honnef.co/go/tools/cmd/staticcheck"
 	_ "mvdan.cc/gofumpt"
 	_ "mvdan.cc/unparam"
