@@ -110,7 +110,8 @@ const download = async () => {
 
         // Don't request the Crowdin API too aggressively to prevent spurious
         // 400 errors.
-        await sleep(400);
+        const sleepTime = process.env.SLEEP_TIME || 250;
+        await sleep(sleepTime);
     }
 
     Promise
